@@ -36,8 +36,10 @@ testit##NAME (void)						\
   DEBUG_NL;							\
 }
 
+#ifndef SKIP_COMPLEX_INT
 T(cc, _Complex char, (0,1))
 T(cs, _Complex short, (1,2))
+#endif
 T(cf, _Complex float, (1.0,2.0))
 
 #undef T
@@ -49,8 +51,10 @@ DEBUG_INIT
 
 #define T(NAME) testit##NAME ();
 
+#ifndef SKIP_COMPLEX_INT
 T(cc)
 T(cs)
+#endif
 T(cf)
 
 DEBUG_FINI
