@@ -25,11 +25,8 @@ Boston, MA 02111-1307, USA.  */
 #include <sys/types.h>
 #include "f2c.h"
 
-#ifdef KR_headers
-integer G77_getpid_0 ()
-#else
-integer G77_getpid_0 (void)
-#endif
+integer
+G77_getpid_0 (void)
 {
   return getpid ();
 }
