@@ -1,4 +1,4 @@
-/* Copyright (C) 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1995, 2001 Free Software Foundation, Inc.
 This file is part of GNU Fortran libU77 library.
 
 This library is free software; you can redistribute it and/or modify it
@@ -20,6 +20,7 @@ Boston, MA 02111-1307, USA.  */
 #include "config.h"
 #endif
 #include <stdio.h>
+#include <sys/types.h>
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -32,18 +33,14 @@ Boston, MA 02111-1307, USA.  */
 #endif
 #include "f2c.h"
 
-#ifdef KR_headers
-/* Subroutine */ int G77_itime_0 (tarray)
-     integer tarray[3];
-#else
-/* Subroutine */ int G77_itime_0 (integer tarray[3])
-#endif
+/* Subroutine */ int
+G77_itime_0 (integer tarray[3])
 {
   struct tm *lt;
   time_t tim;
 
-  tim = time(NULL);
-  lt = localtime(&tim);
+  tim = time (NULL);
+  lt = localtime (&tim);
   tarray[0] = lt->tm_hour;
   tarray[1] = lt->tm_min;
   tarray[2] = lt->tm_sec;
