@@ -2,7 +2,7 @@
 // 2000-11-20
 // Benjamin Kosnik bkoz@redhat.com
 
-// Copyright (C) 2000, 2003 Free Software Foundation, Inc.
+// Copyright (C) 2000, 2003, 2004 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -26,7 +26,7 @@
 void test01()
 {
  using namespace std;
- bool test = true;
+ bool test __attribute__((unused)) = true;
  typedef complex<double> complex_type;
  const double cd1 = -11.451;
  const double cd2 = -442.1533;
