@@ -1,4 +1,4 @@
-// 2004-01-11  Petur Runolfsson  <peturr02@ru.is>
+// 2004-04-16  Petur Runolfsson  <peturr02@ru.is>
 
 // Copyright (C) 2004 Free Software Foundation, Inc.
 //
@@ -18,60 +18,62 @@
 // Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 // USA.
 
-// 27.8.1.4 Overridden virtual functions
-
 #include <fstream>
 #include <locale>
-
 #include <unistd.h>
+#include <signal.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-
 #include <testsuite_hooks.h>
 
-// libstdc++/13582
+// libstdc++/14975
 void test01()
 {
-  bool test __attribute__((unused)) = true;
   using namespace std;
   using namespace __gnu_test;
+  bool test __attribute__((unused)) = true;
 
-  locale loc_en(__gnu_test::try_named_locale("en_US"));
-  locale loc_fr(__gnu_test::try_named_locale("fr_FR"));
+  locale loc_us = try_named_locale("en_US");
 
-  const char* name = "tmp_fifo_13582-2";
-  unlink(name);
+  const char* name = "tmp_14975-2";
+
+  signal(SIGPIPE, SIG_IGN);
+
+  unlink(name);  
   try_mkfifo(name, S_IRWXU);
   
   int child = fork();
+  VERIFY( child != -1 );
+
   if (child == 0)
     {
-      filebuf fbout;
-      fbout.open(name, ios_base::out);
-      fbout.sputn("12345", 5);
-      fbout.pubsync();
+      filebuf fbin;
+      fbin.open(name, ios_base::in);
       sleep(2);
-      fbout.close();
       exit(0);
     }
-
-  wfilebuf fbin;
-  fbin.open(name, ios_base::in);
+  
+  wfilebuf fb;
+  fb.pubimbue(loc_us);
   sleep(1);
-  wfilebuf::int_type n = fbin.sbumpc();
-  VERIFY( n == L'1' );
-  fbin.pubimbue(loc_en);
-  n = fbin.sbumpc();
-  VERIFY( n == L'2' );
-  fbin.pubimbue(loc_fr);
-  n = fbin.sbumpc();
-  VERIFY( n == L'3' );
-  n = fbin.sbumpc();
-  VERIFY( n == L'4' );
-  n = fbin.sbumpc();
-  VERIFY( n == L'5' );
-  n = fbin.sbumpc();
-  VERIFY( n == wfilebuf::traits_type::eof() );
+  wfilebuf* ret = fb.open(name, ios_base::out);
+  VERIFY( ret != NULL );
+  VERIFY( fb.is_open() );
+
+  sleep(3);
+
+  try
+    {
+      fb.sputc(L'a');
+      fb.sputc(L'b');
+      fb.pubimbue(locale::classic());
+      fb.sputc(L'c');
+      fb.close();
+    }
+  catch (std::exception&)
+    {
+    }
 }
 
 int main()
