@@ -27,8 +27,8 @@ int main()
 {
   using namespace __gnu_test;
 
-  // cplus-dem FAIL
-  verify_demangle("_Z1fM1AKFivE", "f(int (A::*)() const)");
+  verify_demangle("_ZlsRKU3fooU4bart1XS2_", 
+		  "operator<<(X bart foo const&, X bart foo const)");
 
   return 0;
 }
